// TEACA BOGDAN
// 335CC

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/io.h>

#include "nokia5110.h"

#define BOARD_ROWS	20
#define BOARD_COLS	10

#define EMPTY 0
#define BOX 1
#define TARGET 2
#define PLAYER 3

enum game_state { START, RUN };
enum game_state state;

uint8_t game_board[BOARD_ROWS * BOARD_COLS];
uint8_t up_button_pressed, down_button_pressed, left_button_pressed, right_button_pressed;

void translate_coords(uint8_t bx, uint8_t by, uint8_t *sx, uint8_t *sy)
{
	*sx = 80 - 4 * bx;
	*sy = 40 - 4 * by;
}

int box1_x = 2, box1_y = 3;
int box2_x = 9, box2_y = 5;
int box3_x = 14, box3_y = 4;
int box4_x = 16, box4_y = 7;

int player_x = 8, player_y = 8;

// function that draws a box on LCD display
void nokia_draw_box(uint8_t x, uint8_t y)
{
	uint8_t colors[4][4] = {
		{1, 1, 1, 1},
		{1, 0 ,0, 1},
		{1, 0, 0, 1},
		{1, 1, 1, 1}
	};

	int i, j;
	for (i = 0 ; i < 4; i++){
		for (j = 0; j < 4; j++){
			nokia_lcd_set_pixel(x + i, y + j, colors[i][j]);
		}
	}
}

// function that draws a target on LCD display
void nokia_draw_target(uint8_t x, uint8_t y)
{
	uint8_t colors[4][4] = {
		{1, 0, 0, 1},
		{0, 1 ,1, 0},
		{0, 1, 1, 0},
		{1, 0, 0, 1}
	};

	int i, j;
	for (i = 0 ; i < 4; i++){
		for (j = 0; j < 4; j++){
			nokia_lcd_set_pixel(x + i, y + j, colors[i][j]);
		}
	}
}

// function that draws the player on LCD display
void nokia_draw_player(uint8_t x, uint8_t y)
{
	uint8_t colors[4][4] = {
		{0, 1, 1, 0},
		{1, 1, 1, 1},
		{1, 1, 1, 1},
		{0, 1, 1, 0}
	};

	int i, j;
	for (i = 0 ; i < 4; i++){
		for (j = 0; j < 4; j++){
			nokia_lcd_set_pixel(x + i, y + j, colors[i][j]);
		}
	}
}

int main(void)
{
	// init buttons
	
	// Left
	DDRB &= ~(1 << PB4);
	PORTB |= (1 << PB4);
	
	// Up
	DDRB &= ~(1 << PB6);
	PORTB |= (1 << PB6);

	// Down
	DDRB &= ~(1 << PB5);
	PORTB |= (1 << PB5);
	
	// Right
	DDRB &= ~(1 << PB3);
	PORTB |= (1 << PB3);
	
	memset(game_board, 0, BOARD_ROWS * BOARD_COLS * sizeof(uint8_t));
	
	// boxes
	game_board[box1_x * BOARD_COLS + box1_y] = BOX;
	game_board[box2_x * BOARD_COLS + box2_y] = BOX;
	game_board[box3_x * BOARD_COLS + box3_y] = BOX;
	game_board[box4_x * BOARD_COLS + box4_y] = BOX;
	
	// targets
	game_board[7 * BOARD_COLS + 7] = TARGET;
	game_board[18 * BOARD_COLS + 4] = TARGET;
	game_board[5 * BOARD_COLS + 5] = TARGET;
	game_board[12 * BOARD_COLS + 1] = TARGET;
	
	// player
	game_board[player_x * BOARD_COLS + player_y] = PLAYER;
	
	state = START;
	
	nokia_lcd_init();
	
	while(1) {
		nokia_lcd_clear();
		
		switch (state) {
		case START:
			// if any button is pressed
			if ((PINB & (1 << PB4)) == 0 || (PINB & (1 << PB5)) == 0 || (PINB & (1 << PB6)) == 0 || (PINB & (1 << PB3)) == 0) {
				state = RUN;
				
				up_button_pressed = 0;
				down_button_pressed = 0;
				left_button_pressed = 0;
				right_button_pressed = 0;
			}
			
			// render start screen
			
			nokia_lcd_set_cursor(0, 46);
			nokia_lcd_write_string("SOKOBAN", 1);
			nokia_lcd_set_cursor(44, 46);
			nokia_lcd_write_string("Start", 1);
			
			break;

		case RUN:
			// check if PB4 (left button) is pressed
			if ((PINB & (1 << PB4)) == 0) {
				if (!left_button_pressed) {
					player_x--;
				
					// check if there is a box on the position the player moved on,
					// and if yes, then move the box to the left
					if(game_board[player_x * BOARD_COLS + player_y] == BOX){
						game_board[(player_x - 1) * BOARD_COLS + player_y] = BOX;
					}
				
					game_board[player_x * BOARD_COLS + player_y] = PLAYER;

					left_button_pressed = 1;
				}
			} else {
				left_button_pressed = 0;
			}

			// check if PB3 (right button) is pressed
			if ((PINB & (1 << PB3)) == 0) {
				if (!right_button_pressed) {
					player_x++;
				
					// check if there is a box on the position the player moved on,
					// and if yes, then move the box to the left
					if(game_board[player_x * BOARD_COLS + player_y] == BOX){
						game_board[(player_x + 1) * BOARD_COLS + player_y] = BOX;
					}
				
					game_board[player_x * BOARD_COLS + player_y] = PLAYER;

					right_button_pressed = 1;
				}
			} else {
				right_button_pressed = 0;
			}
			
			// check if PB6 (up button) is pressed
			if ((PINB & (1 << PB6)) == 0) {
				if (!up_button_pressed) {
					player_y--;
				
					// check if there is a box on the position the player moved on,
					// and if yes, then move the box to the left
					if(game_board[player_x * BOARD_COLS + player_y] == BOX){
						game_board[player_x * BOARD_COLS + player_y - 1] = BOX;
					}
				
					game_board[player_x * BOARD_COLS + player_y] = PLAYER;

					up_button_pressed = 1;
				}
			} else {
				up_button_pressed = 0;
			}
			
			// check if PB5 (down button) is pressed
			if ((PINB & (1 << PB5)) == 0) {
				if (!down_button_pressed) {
					player_y++;
				
					// check if there is a box on the position the player moved on,
					// and if yes, then move the box to the left
					if(game_board[player_x * BOARD_COLS + player_y] == BOX){
						game_board[player_x * BOARD_COLS + player_y + 1] = BOX;
					}
				
					game_board[player_x * BOARD_COLS + player_y] = PLAYER;

					down_button_pressed = 1;
				}
			} else {
				down_button_pressed = 0;
			}
			
			// check if the level is complete (if all boxes are on target positions)
			
			int level_complete = 1;
			
			int i, j;
			
			for (i = 0; i < BOARD_ROWS; i++) {
				int break_out = 0;
				
				for (j = 0; j < BOARD_COLS; j++) {
					if (game_board[i * BOARD_COLS + j] == TARGET) {
						level_complete = 0;
						break_out = 1;
						break;
					}
				}
				
				if(break_out == 1){
					break;
				}
			}
			
			// if level is complete, then go back to start screen
			
			if(level_complete == 1){
				state = START;
				
				continue;
			}
			
			// render game board
			uint8_t x, y;

			for (i = 0; i < BOARD_ROWS; i++) {
				for (j = 0; j < BOARD_COLS; j++) {
					if (game_board[i * BOARD_COLS + j] == BOX) {
						translate_coords(i, j, &x, &y);
						nokia_draw_box(x, y);
					}
					
					if (game_board[i * BOARD_COLS + j] == TARGET) {
						translate_coords(i, j, &x, &y);
						nokia_draw_target(x, y);
					}
					
					if (game_board[i * BOARD_COLS + j] == PLAYER) {
						translate_coords(i, j, &x, &y);
						nokia_draw_player(x, y);
					}
				}
			}
			
			break;
		}
			
		nokia_lcd_render();
	}

	return 0;
}
